@@ -505,6 +505,24 @@ struct TorchDLPackExchangeAPI : public DLPackExchangeAPI {
     }
   }
 
+  // Get current CUDA/ROCm work stream
+  static int CurrentWorkStream(DLDeviceType device_type, int32_t device_id, void** out_stream) {
+    try {
+#ifdef BUILD_WITH_CUDA
+      if (device_type == kDLCUDA || device_type == kDLROCM) {
+        *out_stream = at::cuda::getCurrentCUDAStream(device_id).stream();
+        return 0;
+      }
+#endif
+      // For CPU and other devices, return NULL (no stream concept)
+      *out_stream = nullptr;
+      return 0;
+    } catch (const std::exception& e) {
+      PyErr_SetString(PyExc_RuntimeError, e.what());
+      return -1;
+    }
+  }
+
   static int ManagedTensorToPyObjectNoSync(DLManagedTensorVersioned* src, void** py_obj_out) {
     try {
       at::Tensor tensor = at::fromDLPackImpl<DLManagedTensorVersioned>(src, nullptr);
@@ -531,25 +549,7 @@ struct TorchDLPackExchangeAPI : public DLPackExchangeAPI {
       *out = at::toDLPackImpl<DLManagedTensorVersioned>(tensor);
       return 0;
     } catch (const std::exception& e) {
-      SetError(error_ctx, "TorchDLPackManagedTensorAllocator", e.what());
-      return -1;
-    }
-  }
-
-  // Get current CUDA/ROCm work stream
-  static int CurrentWorkStream(DLDeviceType device_type, int32_t device_id, void** out_stream) {
-    try {
-#ifdef BUILD_WITH_CUDA
-      if (device_type == kDLCUDA || device_type == kDLROCM) {
-        *out_stream = at::cuda::getCurrentCUDAStream(device_id).stream();
-        return 0;
-      }
-#endif
-      // For CPU and other devices, return NULL (no stream concept)
-      *out_stream = nullptr;
-      return 0;
-    } catch (const std::exception& e) {
-      PyErr_SetString(PyExc_RuntimeError, e.what());
+      SetError(error_ctx, "MemoryError", e.what());
       return -1;
     }
   }
@@ -568,6 +568,22 @@ extern "C" DLL_EXPORT int64_t TorchDLPackExchangeAPIPtr() {
 """
 
 
+def parse_env_flags(env_var_name: str) -> list[str]:
+    env_flags = os.environ.get(env_var_name)
+    if env_flags:
+        try:
+            import shlex  # noqa: PLC0415
+
+            return shlex.split(env_flags)
+        except ValueError as e:
+            print(
+                f"Warning: Could not parse {env_var_name} with shlex: {e}. Falling back to simple split.",
+                file=sys.stderr,
+            )
+            return env_flags.split()
+    return []
+
+
 def _generate_ninja_build(
     build_dir: Path,
     libname: str,
@@ -597,7 +613,7 @@ def _generate_ninja_build(
         ]
         default_ldflags = ["/DLL"]
     else:
-        default_cflags = ["-std=c++17", "-fPIC", "-O2"]
+        default_cflags = ["-std=c++17", "-fPIC", "-O3"]
         default_ldflags = ["-shared", "-Wl,-rpath,$ORIGIN", "-Wl,--no-as-needed"]
 
     cflags = default_cflags + [flag.strip() for flag in extra_cflags]
@@ -668,7 +684,7 @@ def main() -> None:  # noqa: PLR0912, PLR0915
     """Build the torch c dlpack extension."""
     # we need to set the following env to avoid tvm_ffi to build the torch c-dlpack addon during importing
     os.environ["TVM_FFI_DISABLE_TORCH_C_DLPACK"] = "1"
-    from tvm_ffi.cpp.load_inline import build_ninja  # noqa: PLC0415
+    from tvm_ffi.cpp.extension import build_ninja  # noqa: PLC0415
     from tvm_ffi.utils.lockfile import FileLock  # noqa: PLC0415
 
     parser = argparse.ArgumentParser(
@@ -791,6 +807,14 @@ def main() -> None:  # noqa: PLR0912, PLR0915
                 ldflags.append(f"-L{python_libdir}")
                 ldflags.append(f"-l{py_version}")
 
+        env_ldflags = parse_env_flags("TVM_FFI_JIT_EXTRA_LDFLAGS")
+        if env_ldflags:
+            ldflags.extend(env_ldflags)
+
+        env_cflags = parse_env_flags("TVM_FFI_JIT_EXTRA_CFLAGS")
+        if env_cflags:
+            cflags.extend(env_cflags)
+
         # generate ninja build file
         _generate_ninja_build(
             build_dir=build_dir,
